# frozen_string_literal: true

require 'net/http'
require 'json'
require 'veezo_location_api/response'

module VeezoLocationApi
  class Client
    class VeezoLocationApiClientError < StandardError; end
    BASE_URI = ENV['VEEZO_LOCATION_API']

    def initialize(tenant_id)
      @tenant_id = tenant_id
    end

    def locations(occurence: Time.now)
      uri = URI(resolve_uri('/locations'))
      body = { occurence: occurence }

      get = generate_get(uri, body)
      request(uri, get)
    end

    private

    def resolve_uri(path)
      "#{BASE_URI}/api/v1/#{@tenant_id}#{path}"
    end

    def generate_get(uri, body = nil)
      get = Net::HTTP::Get.new(uri, 'Content-Type' => 'application/json')
      get['Accept'] = 'application/json'
      get.body = body.to_json if body.is_a?(Hash)
      get
    end

    def request(uri, request)
      http_client = Net::HTTP.new(uri.host, uri.port).tap do |client|
        client.use_ssl = (uri.scheme == 'https')
      end

      response = http_client.request(request)
      response(response)
    rescue => error
      raise(VeezoLocationApiClientError, error)
    end

    def response(response)
      code = response.code.to_i
      success = (code == 200)

      if success
        data = JSON.parse(response.body)
      end

      Response.new(
        success: success, status_code: code, data: data
      )
    end
  end
end